<?php
class PressbackPrincipal {

	function index(){
		global $pressbackup;

		$preferences= get_option('pressbackup_preferences');

		if(!$preferences['pressbackup']['configured'])
		{
			$redirect=array('menu_type'=>'settings', 'controller'=>'settings', 'function'=>'config_init');
		}
		else
		{
			$redirect=array('controller'=>'principal', 'function'=>'dashboard');
		}

		$pressbackup->redirect($redirect);
	}

//------------------------------
//	Pages
//------------------------------

	/*
		@from = [this_site|all_sites], dahboard for this site of for all sites?
	*/

	function dashboard($from = null, $reload=false, $page = null)
	{
		global $pressbackup;
		$pressbackup->import('misc.php');
		$pressbackup->View->layout('principal');
		$preferences= get_option('pressbackup_preferences');

		//check what dashboard we are seeing last time
		if(!$pressbackup->Session->check('dashboard_from')){
			if(!$from){$from='this_site';}
			$pressbackup->Session->write('dashboard_from', $from);
		}
		elseif($from && $pressbackup->Session->read('dashboard_from') != $from)
		{
			$pressbackup->Session->write('dashboard_from', $from);
			$page=1;
		}
		$from = $pressbackup->Session->read('dashboard_from');

		//retrive backups from Amazon S3
		if($preferences['pressbackup']['s3']['credential'])
		{
			$pressbackup->import('S3.php');
			$credentials=explode('|AllYouNeedIsLove|', base64_decode($preferences['pressbackup']['s3']['credential']));
			$s3 = new PressbackupS3($credentials[0], $credentials[1]);
			$bucket_files = $s3->getBucket($preferences['pressbackup']['s3']['bucket_name']);
			$bucket_files= @pressbackup_msort('S3', $bucket_files);
			$bucket_files= filter_files ($from, $bucket_files);
		}

		//retrive backups from Pro
		elseif($preferences['pressbackup']['pressbackuppro']['credential'])
		{
			$pressbackup->import('Pro.php');
			$credentials=explode('|AllYouNeedIsLove|', base64_decode($preferences['pressbackup']['pressbackuppro']['credential']));
			$pbp = new pressbackup($credentials[0], $credentials[1]);
			$bucket_files = $pbp->getFilesList();
			$bucket_files= pressbackup_msort('pressbackup', $bucket_files);
			$bucket_files= filter_files ($from, $bucket_files);
		}

		//inicializacion del paginador
		if (isset($bucket_files) && $bucket_files) {
			$pressbackup->import('paginator.php');
			$pag = new PressbackPaginator();
			$pagination_size = 5;

			$page_saved = $pressbackup->Session->read('dash.page');
			if(!$page && !$page_saved){
				$page = 1;
			}
			elseif(!$page && $page_saved){
				$page = $page_saved;
			}
			if(!array_slice($bucket_files, (($page -1 )*$pagination_size), $pagination_size, true) && $page > 1){
				$page--;
			}
			$pressbackup->Session->write('dash.page', $page);

			$paginator['page'] = $page ;
			$paginator['total']  = count($bucket_files);
			$paginator['pages'] = ceil ($paginator['total'] /$pagination_size);
			$paginator['ini']  = (($page -1 )*$pagination_size);
			$paginator['fin']  = (($page*$pagination_size) -1);
			$paginator['func_path']  = array('controller'=>'principal', 'function'=>'dashboard', $from, null);
			$paginator['pagination'] = $pagination_size;
			$pressbackup->View->set('paginator', $pag->get_html($paginator));
			$pressbackup->View->set('bucket_files', array_slice($bucket_files, $paginator['ini'], $pagination_size, true));
		}
		else{
			$pressbackup->View->set('bucket_files', array());
		}

		//set the type of credential to see what option show in this page
		$credentials = false;
		$credential_type = null;
		if($preferences['pressbackup']['s3']['credential']) {
			$credentials= true; $credential_type='Amazon S3';
		}
		elseif($preferences['pressbackup']['pressbackuppro']['credential']) {
			$credentials= true; $credential_type='Pressbackup Pro';
		}

		//set messages
		if($pressbackup->Session->check('general_msg')){
			$pressbackup->Msg->general($pressbackup->Session->read('general_msg'));
			$pressbackup->Session->delete('general_msg');
		}

		//set errors
		if($pressbackup->Session->check('error_msg')){
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error_msg');
		}

		//set errors de permisos
		if($pressbackup->Session->check('error_msg_perm')){
			$pressbackup->View->set('perm_problem', $pressbackup->Session->read('error_msg_perm'));
			$pressbackup->Session->delete('error_msg_perm');
		}

		//check for activated schedule
		if (!$ns=wp_next_scheduled('pressback_backup_start_cronjob')) {
			$pressbackup->import('backup_functions.php');
			$pb = new pressbackup_backup();
			$pb->add_schedule('+1 minutes');
			$ns=wp_next_scheduled('pressback_backup_start_cronjob');
		}

		//check for activated schedule
		$cgi=false;
		if (strpos(php_sapi_name(), 'cgi') !== false) {
			$cgi=true;
		}

		$pressbackup->View->set('from', $from);
		$pressbackup->View->set('settings', $preferences);
		$pressbackup->View->set('credentials', $credentials);
		$pressbackup->View->set('credential_type', $credential_type);
		$pressbackup->View->set('ns', $ns);
		$pressbackup->View->set('cgi', $cgi);
		$pressbackup->View->set('reload', $reload);
	}

	function backup_upload_page ()
	{
		//tools & info
		global $pressbackup;
		$pressbackup->import('misc.php');
		$pressbackup->View->layout('principal');
		$preferences= get_option('pressbackup_preferences');

		//set error from previus upload
		if($pressbackup->Session->check('error_msg')){
			$pressbackup->Msg->error($pressbackup->Session->read('error_msg'));
			$pressbackup->Session->delete('error_msg');
		}

		//check for folder permissions
		$permissions = array();
		$disable = false;
		$msg = false;

		$dir = WP_CONTENT_DIR . DS;
		if( substr(sprintf('%o', @fileperms( $dir . 'themes')), -3) < '777'){ $permissions[] ='themes'; }
		if( file_exists($dir . 'uploads') && substr(sprintf('%o', @fileperms( $dir . 'uploads')), -3) < '777'){ $permissions[] ='uploads'; }
		if( substr(sprintf('%o', @fileperms( $dir . 'plugins')), -3) < '777'){ $permissions[] ='plugins'; }

		if($permissions) {
			$msg = "Warning: permissions denied to write on <b>".join('</b>, <b>', $permissions)."</b> folder/s, change the permissions if you want to be able to restore a backup";
			$disable = true;
		}

		$pressbackup->View->set('upload_size', pressbackup_get_size_in('m', pressbackup_upload_max_filesize()));
		$pressbackup->View->set('settings', $preferences);
		$pressbackup->View->set('disblable_msg', $msg);
		$pressbackup->View->set('disable_ulpoad', $disable);
	}

	function host_info(){
		global $pressbackup;

		$wp['version'] = get_bloginfo ('version');
		$wp['wpurl'] = get_bloginfo ('wpurl');
		$pressbackup->View->set('modules',  get_loaded_extensions());
		$pressbackup->View->set('server',  $_SERVER);
		$pressbackup->View->set('server_sapi',  php_sapi_name());
		$pressbackup->View->set('wp',  $wp);
		$pressbackup->View->set('p',  get_plugin_data($pressbackup->main_file));
		$pressbackup->View->set('psystmp', $pressbackup->Path->Dir['SYSTMP']);
		$pressbackup->View->set('pwpftmp', $pressbackup->Path->Dir['SYSTMP']);
	}

//------------------------------
//	functions
//------------------------------


	function backup_start_createsend() {
		@ignore_user_abort(true);
		@set_time_limit(0);

		//tools
		global $pressbackup;
		$pressbackup->import('backup_functions.php');
		$preferences= get_option('pressbackup_preferences');
		$pb = new pressbackup_backup();

		//get the type of credential
		$credential = 'Pro';
		if($preferences['pressbackup']['s3']['credential']) {
			$credential='S3';
		}

		if(!$file = $pb->create()){ exit(); }
		$pb->save_on($credential, $file);
		exit;
	}

	function backup_create($reload_type="backup_download") {
		//tools
		global $pressbackup;
		$pressbackup->import('backup_functions.php');
		$pressbackup->import('curl.php');
		$preferences= get_option('pressbackup_preferences');

		if(strpos(php_sapi_name(), 'cgi') !== FALSE ) {

			$info = array();
			$cookie = array(); foreach($_COOKIE as $key => $value) { $cookie[]=$key.'='.$value;}

			if ($reload_type == "backup_download") {
				//create backup file
				$info[0] = get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup_backup_create_zip';
				$info[1]= join (';', $cookie);
			}else{
				$info[0] = get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup_backup_createsend';
				$info[1]= join (';', $cookie);
			}

			$output = array();
			$return_var = 1;
			$path= 'php '.$pressbackup->Path->Dir['LIB'].DS.'background.php'.' "ALLYOUNEEDISLOVE" "'.base64_encode(join('|ALLYOUNEEDISLOVE|', $info)).'" > /dev/null 2>&1 & echo $!';
			echo $path;
			exec ($path, $output, $return_var);

		}else{

			$curl = new PressbackupCurl();
			if ($reload_type == "backup_download") {
				//create backup file
				$args = array(
					'url' => get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup_backup_create_zip',
					'cookie'=>$_COOKIE,
					'timeout'=>3,
				);
				$curl->call($args);
			}else{
				$args = array(
					'url' => get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup_backup_createsend',
					'cookie'=>$_COOKIE,
					'timeout'=>3,
				);
				$curl->call($args);
			}

		}

		$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard', null, $reload_type));
		exit();
	}

	function backup_create_zip () {
		@set_time_limit(0);

		//tools
		global $pressbackup;
		$pressbackup->import('backup_functions.php');

		$pb = new pressbackup_backup();
		if(!$zip_file_created = $pb->create()){
			exit;
		}
		@file_put_contents($pressbackup->Path->Dir['LOGTMP'] . DS . 'create.return', $zip_file_created);
		exit;
	}

	function backup_download ($file =null){
		global $pressbackup;
		@ob_end_clean();
		$pressbackup->View->layout('blank');
		$pressbackup->import('download.php');
		$pressbackup->View->set('file', base64_decode($file));
	}

	function backup_upload ($then= 'restore')
	{
		//tools & info
		global $pressbackup;
		$preferences= get_option('pressbackup_preferences');
		$pressbackup->import('backup_functions.php');

		//return if exists upload errors
		if(!$this->pressbackup_check_backup_upload() || !$this->pressbackup_check_backup_integrity($_FILES['backup']['tmp_name']))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'backup_upload_page'));
		}

		//create backup file
		$pb = new pressbackup_backup();
		$pb->restore(array('tmp_name'=>$_FILES['backup']['tmp_name']));
		$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
	}

	function backup_restore ($name= null)
	{
		//tools & info
		global $pressbackup;
		$preferences= get_option('pressbackup_preferences');
		$pressbackup->import('backup_functions.php');

		//check for folder permissions
		$permissions_problem = array();
		$dir = WP_CONTENT_DIR . DS;
		if( substr(sprintf('%o', @fileperms( $dir . 'themes')), -3) < '777'){ $permissions_problem[] ='themes'; }
		if( file_exists($dir . 'uploads') && substr(sprintf('%o', @fileperms( $dir . 'uploads')), -3) < '777'){ $permissions_problem[] ='uploads'; }
		if( substr(sprintf('%o', @fileperms( $dir . 'plugins')), -3) < '777'){ $permissions_problem[] ='plugins'; }

		if($permissions_problem) {
			$msg = "Warning: permissions denied to write on <b>".join('</b>, <b>', $permissions_problem)."</b> folder/s, change the permissions if you want to be able to restore a backup";
			$pressbackup->Session->write( 'error_msg_perm', $msg);
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}

		$pb = new pressbackup_backup();
		if(!$tmp_name=$pb->get($name))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}
		elseif(!$this->pressbackup_check_backup_integrity($tmp_name))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}

		$pb->restore(array('tmp_name'=>$tmp_name));
		$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
	}

	function backup_get ($name=null)
	{
		//tools & info
		global $pressbackup;
		$preferences= get_option('pressbackup_preferences');
		$pressbackup->import('backup_functions.php');

		$pb = new pressbackup_backup();
		if(!$tmp_name=$pb->get($name))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}
		elseif(!$this->pressbackup_check_backup_integrity($tmp_name))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}

		ob_end_clean();
		$pressbackup->import('download.php');
		$pressbackup->View->set('file', $tmp_name);
	}

	function backup_get2 ($name=null)
	{
		//tools & info
		global $pressbackup;
		$preferences= get_option('pressbackup_preferences');
		$pressbackup->import('backup_functions.php');

		$pb = new pressbackup_backup();
		if(!$tmp_name=$pb->get2($name))
		{
			$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
		}

		$pressbackup->redirect('http://pressbackup.com/pro/api/download/'.base64_encode($name));
	}

	function backup_delete ($name=null)
	{
		//tools & info
		global $pressbackup;
		$preferences= get_option('pressbackup_preferences');
		$pressbackup->import('backup_functions.php');

		$pb = new pressbackup_backup();
		$pb->delete($name);
		$pressbackup->redirect(array('controller'=>'principal', 'function'=>'dashboard'));
	}

//------------------------------
//	functions helpers
//------------------------------


	function pressbackup_check_backup_upload()
	{
		//tools
		global $pressbackup;
		$pressbackup->import('misc.php');

		if (!isset($_FILES['backup']) || $_FILES['backup']['error']==4)
		{
			$pressbackup->Session->write( 'error_msg', 'You sent an empty file or it is bigger than '.pressbackup_get_size_in('m', pressbackup_upload_max_filesize()).' Mb. Please try it again');
			return false;
		}
		if ($_FILES['backup']['error']!=0)
		{
			$pressbackup->Session->write( 'error_msg', 'There was a problem, maybe the file is bigger than '.pressbackup_get_size_in('m', pressbackup_upload_max_filesize()).' Mb. Please try it again');
			return false;
		}
		if ( !in_array($_FILES['backup']['type'], array('application/zip', 'application/x-zip-compressed')))
		{
			$pressbackup->Session->write( 'error_msg', 'Wrong file type: '.$_FILES['backup']['type']);
			return false;
		}
		if (!is_uploaded_file($_FILES['backup']['tmp_name']))
		{
			$pressbackup->Session->write( 'error_msg', 'The file could not be uploaded correctly');
			return false;
		}
		return true;
	}

	function pressbackup_check_backup_integrity($zip_file)
	{
		//tools
		global $pressbackup;

		if(!file_exists($zip_file)){
			$pressbackup->Session->write( 'error_msg', 'file not found');
			return false;
		}

		$zip = new ZipArchive();
		if ($zip->open($zip_file) !== TRUE)
		{
			$pressbackup->Session->write( 'error_msg', 'Sorry, but the file is corrupt');
			return false;
		}
		$zip->close();
		return true;
	}

	function pressbackup_check_create_send_status()
	{
		//tools
		global $pressbackup;

		$task = $_POST['task'];

		if(file_exists( $logfilec = $pressbackup->Path->Dir['LOGTMP']. DS. 'create.log') && !file_exists( $logfiles = $pressbackup->Path->Dir['LOGTMP']. DS. 'sent.log') ) {

			$get = file_get_contents($logfilec);
			switch($get){
				case 'fail':
					@unlink($logfilec);
					exit('reload|fail');
				break;
				case 'finish':
					$rget='';
					if(file_exists( $returnfile = $pressbackup->Path->Dir['LOGTMP']. DS. 'create.return')) { $rget= file_get_contents( $returnfile ); }
					@unlink($logfilec);
					@unlink($returnfile);
					if($task == 'download') {
						exit('reload|'.base64_encode($rget));
					}
					else{
						exit('wait|progress_img|Start Sending');
					}
				break;
				default:
					exit('wait|progress_img|Crating Backup');
				break;
			}
		}
		elseif(file_exists( $logfiles = $pressbackup->Path->Dir['LOGTMP']. DS. 'sent.log') ) {

			@unlink($logfilec);
			$get = file_get_contents($logfiles);
			switch($get){
				case 'fail':
					@unlink($logfiles);
					$pressbackup->Session->delete( 'sent.percent');
					exit('reload|fail');
				break;
				case 'finish':
					$rget='';
					if(file_exists( $returnfile = $pressbackup->Path->Dir['LOGTMP']. DS. 'sent.return')) { $rget= file_get_contents( $returnfile ); }
					@unlink($logfiles);
					@unlink($returnfile);
					$pressbackup->Session->delete( 'sent.percent');
					exit('reload|'.base64_encode($rget));
				break;
				default:
					exit('wait|progress_bar|'.$pressbackup->Session->read( 'sent.percent').'|Sending Backup');
				break;
			}
		}
		exit('wait|pfail');
	}
}
?>
